﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Text.RegularExpressions;
using System.Net;

namespace XTX.parser
{
    public partial class Engine
    {
        [RegisterAction()]
        private object regex(XmlNode e)
        {
            Regex r = new Regex(e.Attributes["reg"].Value, RegexOptions.IgnoreCase | RegexOptions.Multiline);
            return r.Matches(cast<string>(handle(xmlFirstChild(e))));
        }

        [RegisterAction()]
        private string concat(XmlNode e)
        {
            string r = "";
            foreach (XmlNode z in e)
            {
                if (!skipNode(z))
                    r += cast<string>(handle(z));
            }
            return r;
        }

        [RegisterAction()]
        private object first(XmlNode e)
        {
            object r = handle(xmlFirstChild(e));
            if (r is IEnumerable)
            {
                return first((IEnumerable)r);
            }
            throw new Exception("Can not apply `first` to instance of `" + r.GetType().ToString() + "`");
        }

        private object first(IEnumerable e)
        {
            IEnumerator i = e.GetEnumerator();
            if (i.MoveNext())
                return i.Current;
            else
                return null;
        }

        [RegisterAction(), RegisterAction("t")]
        private string text(XmlNode e)
        {
            return value(e.FirstChild);
        }

        [RegisterAction("#text")]
        private string value(XmlNode e)
        {
            if (e == null)
                return "";
            return e.Value;
        }

        [RegisterAction()]
        private string input(XmlNode e)
        {
            return input_string;
        }

        [RegisterAction()]
        private string download(XmlNode e)
        {
            string url = cast<string>(handle(xmlFirstChild(e)));
            return (new WebClient()).DownloadString(url);
        }

        [RegisterAction()]
        private string shrink_whitespaces(XmlNode e)
        {
            string a = cast<string>(handle(xmlFirstChild(e)));
            return Regex.Replace(a.Replace("\r", " ").Replace("\n", " ").Replace("\t", " "), " +", " ");
        }

        [RegisterAction()]
        private string replace(XmlNode e)
        {
            string from = cast<string>(handle(extractParam(e, "from")));
            string what = cast<string>(handle(extractParam(e, "what")));
            string to = cast<string>(handle(extractParam(e, "to")));
            bool use_regex = extractBoolAttribute(e, "use_regex", false);
            if (use_regex)
                return Regex.Replace(what, from, to);
            else
                return what.Replace(from, to);
        }

        [RegisterAction()]
        private Group group(XmlNode e)
        {
            object g = handle(xmlFirstChild(e));
            int i = int.Parse(e.Attributes["index"].Value);
            Match c = cast<Match>(g);
            return c.Groups[i];
        }

        [RegisterAction()]
        private string cut_bottom(XmlNode e)
        {
            string c = cast<string>(handle(xmlFirstChild(e)));
            string s = e.Attributes["from"].Value;
            int i = c.IndexOf(s);
            if (i > 0)
                return c.Substring(0, i);
            return c;
        }
    }
}